<?php defined('BASEPATH') OR exit('No direct script access allowed');

class OT_App {

	private $entity;

	function __construct()
	{
		$this->CI	= & get_instance();
		$this->CI->load->library('user_agent');

		// Check access via ajax or browser
		if ($this->CI->agent->is_browser())
		{
			// load from session if exists
			if ($result = $this->CI->session->userdata('entity'))
			{
				$this->entity = $result;
			}
		}

		if (!$this->entity || $this->CI->session->userdata('lic_invalid'))
		{
			$this->_load();
		}

		if ($this->entity)
		{
			if ($user		= $this->CI->session->userdata('user'))
			{
				$task_count	= $this->get_task_count($this->entity->entity_id, $user['user_id']);

				$this->CI->session->set_userdata(array(
					'task_count'	=> $task_count,
					'score'	=> (!empty($score) && $score['score'] ? $score['score'] : ''),
					'today_score'	=> (!empty($today_score) && $today_score['score'] ? $today_score['score'] : '')
				));
			}

			if (!defined('CURRENCY'))
			{
				define('CURRENCY_CODE', $this->entity->currency_code);
				define('CURRENCY', $this->entity->currency_symbol);
			}
			if (!defined('DATE_FORMAT'))
			{
				define('DATE_FORMAT', $this->entity->date_format);
				$dtFormat	= $this->entity->date_format;
				$dtFormat	= ($dtFormat ? $dtFormat : 'MMM dd, yyyy');
				define('DATE_FORMAT_DISPLAY', $this->CI->config->config['date_formats'][$this->entity->date_format]);
			}
			if (!defined('START_WEEK_ON'))
			{
				define('START_WEEK_ON', $this->entity->start_week_on);
			}
			if (!defined('NON_WORKING_DAYS'))
			{
				define('NON_WORKING_DAYS', $this->entity->non_working_days);
			}
			if (!defined('NUMBER_FORMAT'))
			{
				define('NUMBER_FORMAT', $this->entity->number_format);
			}
		}
	}

	private function get_task_count($entity_id=0, $user_id=0)
	{
		$this->CI->db->select('pt.task_id');
		$this->CI->db->from('project_task AS pt');
//		$this->CI->db->join('project_task_users AS ptu', 'ptu.task_id=pt.task_id');
		$this->CI->db->where(array(
			'pt.entity_id'	=> $entity_id,
			'pt.is_closed'	=> '0',
			'pt.assignee_id'	=> $user_id
		));

		$query	= $this->CI->db->get();

		return count($query->result_array());
	}

	// load entity based on subdomain
	public function _load()
	{
        $subdomain = $_SERVER['HTTP_HOST']; //assigns the first part

		$result	= array();
		// check for subdomain
		$ignore_domains	= array('localhost','www');
		if ($subdomain)
		{
			$this->CI->db->select('e.entity_id, e.name, ea.is_active, es.time_zone_code, es.currency_code, es.start_week_on, es.date_format, es.period_format, es.number_format, es.time_format, es.time_stamp_req,es.time_sheet_approval, es.modify_billable, es.non_working_days, es.time_round_to, es.hours_per_day, e.entity_logo, es.skip_daysoff, es.fiscal_year_month, ea.gdomain');
			$this->CI->db->from('entity AS e');
			$this->CI->db->join('entity_accounts AS ea', 'ea.entity_id = e.entity_id', 'inner');
			$this->CI->db->join('entity_setting AS es','es.entity_id = e.entity_id', 'inner');
//			$this->CI->db->where('ea.is_active', 1);

			if (!defined('APP_EXP') || in_array($subdomain, $ignore_domains))
			{
				//	$this->CI->db->where(array('e.entity_id' => 1));
				$this->CI->db->limit(1);
			}
			else
			{
				$this->CI->db->where("FIND_IN_SET('$subdomain',ea.domain_url) !=", 0);
			}

			$query	= $this->CI->db->get();

			if ($query->num_rows())
			{
				$result	= $query->row();

				$query	= $this->CI->db->select('currency_symbol')->from('currency')->where('currency_code', $result->currency_code)->get();
				if ($query->num_rows())
				{
					$currency	= $query->row();
					$result->currency_symbol	= $currency->currency_symbol;

					$this->CI->db->select('description, repeat');
					$this->CI->db->select("DATE_FORMAT(dayoff_date, '%Y-%m-%d') as dayoff_date", false);
					$this->CI->db->from('entity_daysoff');
					$this->CI->db->where(array('entity_id' => $result->entity_id));
					$query = $this->CI->db->get();
					$result->dayoff = $query->result_array();
				}
			}
		}
		$this->CI->session->set_userdata('entity', $result);

		if ($this->CI->session->userdata('user')) {
			$this->CI->load->library('auth');
			$entity = $this->CI->auth->license_data();
//			$this->CI->session->set_userdata('lic_invalid',1);

			if (is_object($entity))  {
				$result = $entity;
				$this->CI->session->set_userdata('entity',$entity);
//				$this->CI->session->set_userdata('lic_invalid',0);
			}
		}
		$this->entity = $result;
	}

	// check valid domain
	public function valid()
	{
		$this->CI->load->response();
		// check domain exists
		if (!$this->entity)
		{		
			show_404(lang('DOMIAN_NOT_EXISTS'), 404);
			return false;
		}

		// check domain is active
		if ($this->entity->is_active!=1)
		{
			show_error(lang('DOMIAN_NOT_ACTIVE'));
			return false;
		}

		return true;
	}

	/**
	* Returns a property of the object or the default value if the property is not set.
	*/
	public function get($property, $default = null)
	{
		if (isset($this->entity->$property))
		{
			return $this->entity->$property;
		}
		return $default;
	}
}
