<?php

if (!defined('BASEPATH'))
    exit('No direct script access allowed');

require_once(FCPATH . '/application/libraries/base_table.php');

class Invoice_table extends Base_table {

	function __construct()
	{
        parent::__construct(array('tbl'=>'invoice', 'tbl_key'=>'invoice_id'));
	}
		
	public function formatdata(&$data) 
	{
		$this->CI->load->model('invoices');
		$data['invoice_number'] = !empty($data['invoice_number'])?$data['invoice_number']:$this->CI->invoices->get_invoice_defaults();
		
		// unset tax percentage if it disabled
		if(isset($data['tax1_enable']))
			$data['tax1_percentage'] = ($data['tax1_enable'] && isset($data['tax1_percentage']))?$data['tax1_percentage']:0;
		if(isset($data['tax2_enable']))
			$data['tax2_percentage'] = ($data['tax2_enable'] && isset($data['tax2_percentage']))?$data['tax2_percentage']:0;
		if(isset($data['tax3_enable']))
			$data['tax3_percentage'] = ($data['tax3_enable'] && isset($data['tax3_percentage']))?$data['tax3_percentage']:0;

		if (!empty($data['imported_data'])) {
			$import_config = array();
			
			if (!empty($data['imported_data']['project_ids'])) {
				$import_config['ids']	= $data['imported_data']['project_ids'];
			}
			/*$diff_day 	= 0;
			$to_date	= $data['to_date'];

			$import_config['filter'] = array($data['date_filter_type']);
			if ($data['date_filter_type']=='CU') {
				$diff_day = (strtotime($data['to_date'])-strtotime($data['from_date']))/(24*60*60);
				$to_date	= date('Y-m-d', (strtotime($data['to_date']) + (24*60*60)));
				$import_config['filter'] = array($data['date_filter_type'], $to_date,$diff_day);
			}*/
			if (!empty($data['imported_data']['invoice_hours'])) {
				$import_config['hours'] = array_values($data['imported_data']['group_by']);
			}
			if (!empty($data['imported_data']['invoice_expenses'])) {
				$import_config['expenses'] = $data['imported_data']['detail_expenses'];
			}
			$data['import_config'] = JSON_encode($import_config);
		}

        return true;
    }
 
    function save(&$data, $params = array())
    {
		$result = array();
    	$values = $data;

		if ($id = parent::save($values, $params))
		{
		
			// save items
            if (!empty($data['details']))
            {
            	foreach ($data['details'] as $key => $item)
            	{
            		$item['invoice_id']	= $id;
            		$result['items'][] = $this->save_items($item);
            	}
            }

            //delete items
			if (!empty($data['deleted']))
            {
            	$this->delete_seleted_items($data['deleted']);
            }
            
            // estimate total calcuation
       		$this->set_invoice_total($id, $data,$params);
				
       		// Insert invoice activity
			$fullname = $this->CI->ot_user->get('first_name').' '.$this->CI->ot_user->get('last_name').' ';
			$currency_symbol = get_currency($data['currency_code']);
			$amount =  $data['amount'];
			if(isset($data['estimate_id']))
			$amount = (!isset($data['details'][0]['estimate_id']))? $data['details'][0]['amount'] : $data['amount'];
			$act_data = array(
					'invoice_id' 	=> $id,					
					'a_date'		=> $this->cur_time,
					'a_mode'		=> (isset($data['invoice_id']) && $data['invoice_id'] > 0)?'IU':'IN',
					'a_notes'		=> (isset($data['invoice_id']) && $data['invoice_id'] > 0) ? $fullname .' updated invoice #'.$data['invoice_number'].' ('.$currency_symbol.' '.$amount.')' : $fullname. ' created invoice #'.$data['invoice_number'].' ('.$currency_symbol.' '.$amount.')'
			);
								
			if (! invoice_activity($act_data))
			{
					return false;
			}					

            $result['invoice_id'] = $id;
            $result['invoice_number'] = $data['invoice_number'];
		}
        return $result;
    }
     
	public function set_invoice_total ($invoice_id, $data, $params = array())
	{
		// get invoice related items
		$this->CI->load->model('invoices');
		$details = $this->CI->invoices->get_invoice_details($invoice_id);
		
		if ($details)
		{
			$amount = 0;
			// calculate amount
			foreach ($details as $key => $row)
			{
				$details[$key]['amount'] = $row['quantity']*$row['rate'];
				$amount+=$details[$key]['amount']; 
			}
			
			//discount calculation
			$discount_percentage = !empty($data['discount_percentage'])?($data['discount_percentage']/100):0;
			if (!$discount_percentage && $amount)
			{
				$discount_percentage = !empty($data['discount_amount'])?$data['discount_amount']/$amount:0;		
			}
			$data['discount_amount'] = ($discount_percentage)?($amount*$discount_percentage):0;
			$amount	-= $data['discount_amount'];
			
			// calculate tax
			$tax = 0;
			$data['tax1_amount'] = 0;
			$data['tax2_amount'] = 0;
			$data['tax3_amount'] = 0;
			
			foreach ($details as $key => $row)
			{
				$tmp_amount	 = $row['amount'];
				$tmp_amount	-= ($discount_percentage)?($tmp_amount*$discount_percentage):0;
				if ($row['is_tax1'])
				{
					$data['tax1_amount']	+= ($tmp_amount*($data['tax1_percentage']/100));
				}
				if ($row['is_tax2'])
				{
					$data['tax2_amount']	+= $tmp_amount*($data['tax2_percentage']/100);
				}
				if ($row['is_tax3'])
				{
					$data['tax3_amount']	+= $tmp_amount*($data['tax3_percentage']/100);
				}
			}

			$update_data['tax1_amount'] = floatval($data['tax1_amount']);
			$update_data['tax2_amount'] = floatval($data['tax2_amount']);
			$update_data['tax3_amount'] = floatval($data['tax3_amount']);
			$update_data['amount']		= round($amount + $update_data['tax1_amount'] + $update_data['tax2_amount'] + $update_data['tax3_amount'], 2);

			if (isset($params['where']) && $params['where'])
			{
				$where = $params['where'];
			}
			$where['invoice_id'] = $invoice_id;
			
			$this->CI->db->where($where);
			if (isset($params['where_str']) && $params['where_str'])
			{
				$this->CI->db->where($params['where_str']);
			}
			
			$this->CI->db->update('invoice', $update_data);			
		}
		
		return true;
	}
    
    function save_items ($data)
    {
		$values				= $data;
		$values['tbl_key']	= 'invoice_item_id';
		$values['tbl']		= 'invoice_items';
		
		$params=array();
   		if (!empty($data[$values['tbl_key']]))
		{
			$params['where'] = array('invoice_id' => $data['invoice_id']);
		} else {
			$this->CI->db->select_max('serial_no');
			$query = $this->CI->db->get_where($values['tbl'], array('invoice_id' => $data['invoice_id']));
			$tmp = $query->row();
			$values['serial_no'] = isset($tmp->serial_no)?$tmp->serial_no+1:1; 
		}
		
		$id = parent::save($values, $params);

		return $id;
    }
 
	public function delete_seleted_items ($ids)
	{
		// delete items
		$this->CI->db->where_in('invoice_item_id',$ids);
		$this->CI->db->delete('invoice_items');
		
		return true;
	}
		
 	function payment_save ($data)
    {
    	$values = $data;
		$values['tbl_key']	= 'invoice_payment_id';
		$values['tbl']		= 'invoice_payment';
		$invoice_no = $values['invoice_number'];

		// get current invoice detail
    	$where = array('inv.entity_id' => $this->entity_id, 'inv.invoice_id' => $values['invoice_id']);
    	$dep_payment = $params=array();
    	$join = 'inv.entity_id=invp.entity_id AND inv.invoice_id = invp.invoice_id';
		if (!empty($values[$values['tbl_key']]))
		{
			$join.= ' AND invp.invoice_payment_id != '. $values[$values['tbl_key']];
			$params['where'] = array($values['tbl_key'] => $values[$values['tbl_key']], 'invoice_id' => $values['invoice_id']);
		}
		
		$this->CI->db->select('inv.invoice_id, inv.amount,inv.is_deposit, inv.currency_code, inv.status, sum(invp.amount) as  payment_amount', false);
		$this->CI->db->from('invoice AS inv');
		$this->CI->db->join('invoice_payment AS invp', $join, 'left');
		$this->CI->db->where($where);		
		$this->CI->db->group_by('invp.invoice_id');
		
		$query = $this->CI->db->get();
		$invoice = $query->row_array();
		
		// get sum of payment amount
		$update = array();		
		$update['payment_amount']	= !empty($invoice['payment_amount'])?($invoice['payment_amount']+$values['amount']):$values['amount'];
		$update['status'] = ($invoice['amount']==$update['payment_amount'])?'p':'b';
		$update['last_updated_by']	= $this->user_id;
		$update['last_updated_date']	= $this->cur_time;
		
		// validate amount
		if ($update['payment_amount']>$invoice['amount'])
		{
			return false;
		}
		
		// check deposit invoice balance if payment through deposit invoice
		if (!empty($values['invoice_payment_id']) && $values['deposit_invoice_id'])
		{
			$this->CI->db->select('inv.invoice_id, inv.amount,inv.is_deposit, inv.status, inv.payment_amount, sum(invp.amount) as  deposit_used', false);
			$this->CI->db->from('invoice AS inv');
			$this->CI->db->join('invoice_payment AS invp', 'inv.entity_id=invp.entity_id and inv.deposit_invoice_id = invp.invoice_id AND invp.invoice_payment_id != '. $values[$values['tbl_key']], 'left');
			$this->CI->db->where(array('inv.invoice_id' => $values['deposit_invoice_id']));
			$this->CI->db->group_by('invp.invoice_id');
			
			$query = $this->CI->db->get();
			$dep_payment = $query->row_array();
			
			$deposit_used = !empty($dep_payment['deposit_used'])?($dep_payment['deposit_used']+$values['amount']):$values['amount'];
			
			// validate amount
			if ($deposit_used > $dep_payment['payment_amount'])
			{
				return false;
			}
			$deposit_balance = $dep_payment['payment_amount'] - $deposit_used;
			$this->CI->db->where(array('invoice_id' => $values['deposit_invoice_id']));
			$this->CI->db->update('invoice', array('deposit_balance' => $deposit_balance));
		}
		
		$inv_pay_id = parent::save($values, $params);
		
		// reset deposit invoice when edited payment paid via deposit invoice
		if (isset($dep_payment['deposit_balance']))
		{						
			$this->CI->db->where(array('invoice_id' => $values['deposit_invoice_id']));
			$this->CI->db->update('invoice', array('deposit_balance' => $dep_payment['deposit_balance']));
		}
		// update deposit balance if it deposit invoice
		if ($invoice['is_deposit'])
		{
			$row = $this->get_deposit_used($values['invoice_id']);	
			$update['deposit_balance'] = $update['payment_amount'] - (!empty($row['deposit_used'])?$row['deposit_used']:0);
		}
		
		$this->CI->db->where(array('invoice_id' => $values['invoice_id']));
		$this->CI->db->update('invoice', $update);
		
    	// Insert invoice activity		
		$fullname = $this->CI->ot_user->get('first_name').' '.$this->CI->ot_user->get('last_name');
		$currency_symbol = get_currency($invoice['currency_code']);
		$act_data = array(
				'invoice_id' 	=> $values['invoice_id'],
				'a_mode'		=> 'IP',
				'a_notes'		=>  $fullname. ' updated payment for Invoice #'.$invoice_no.'. '.$currency_symbol.' '.$values['amount'].' of '.$currency_symbol.' '.$invoice['amount'].' is paid'
		);		
		if (! invoice_activity($act_data))
		{
				return false;
		}
		
		return array('invoice_payment_id' => $inv_pay_id, 'status' => $update['status']);
    }
    
    function get_deposit_used($invoice_id, $where = array())
    {
    	$this->CI->db->select('sum(amount) as deposit_used');
		$this->CI->db->from('invoice_payment');
		$where['deposit_invoice_id'] = $invoice_id;
		$this->CI->db->where($where);
		$this->CI->db->group_by('deposit_invoice_id');
		$query = $this->CI->db->get();
		$row = $query->row_array();
		
		return $row;
    }	   
}
